
data zips;
input zip @@;
datalines;
12037 20001 10001 02138  97201
;

/*data zips;*/
/*set zips;*/
/*do j=1 to 100000; */
/*   output;*/
/*end;*/
/*run;*/

* find CITY, STATE, LONGITUDE/LATITUDE of ZIP centroid;

%c(sashelp.zipcode);

* data step merge;
proc sort data=zips out=z1;
by zip;
run;

proc sort data=sashelp.zipcode (keep=zip city statecode x y) out=z2;
by zip;
run;

data z3;
merge z1 (in=a) z2;
by zip;
if a;
run;

%p(z3);

%c(z3);

* SQL;
proc sql;
create table z4 as
select a.zip, city, statecode, x, y
from zips as a, sashelp.zipcode as b
where a.zip = b.zip;
quit;

%p(z4);

%c(z4);

* format;
data temp / view=temp;
retain fmtname 'z2info';
set sashelp.zipcode;
start = zip;
label = catx(',',city, statecode,x,y);
run;

proc format cntlin=temp;
run;

data z5;
set zips;
info = put(zip,z2info.);
if cat(zip) ne info; 
city = scan(info,1,',');
statecode = scan(info,2,',');
x = input(scan(info,3,','),12.);
y = input(scan(info,4,','),12.);
drop info;
run;

%p(z5);

%c(z5);

* INDEX;
%c(sashelp.zipcode,s=);

data z6;
set zips;
set sashelp.zipcode (keep=zip city statecode x y) key=zip;
run;

%p(z6);

%c(z6);

data zips (index=(zip));
input zip @@;
datalines;
12037 20001 10001 02138  97201
;

data z0;
merge zips (in=a) sashelp.zipcode (keep=zip city statecode x y);
by zip;
if a;
run;

%p(z0);

data cities (index=(city2_statecode=(city2 statecode)));
input city2 : $35. statecode : $2.;
datalines;
CHATHAM NY
CAMBRIDGE MA
PORTLAND OR
WASHINGTON DC
;

%c(cities,s=);

data c1;
merge cities (in=a) sashelp.zipcode (keep=zip city2 statecode x y);
by city2 statecode;
if a;
run;

%p(c1);

* HASH TABLE;
data z7;
length city $35 statecode $2;
declare hash z(dataset: 'sashelp.zipcode');
z.definekey('zip');
z.definedata('city', 'statecode', 'x', 'y');
z.definedone();
call missing(city, statecode, x, y);
do until(last);
   set zips end=last;
   if z.find() eq 0 then output;
end;
run;

%p(z7);

%c(z7);

data z7;
set sashelp.zipcode (keep=city statecode x y);
declare hash z(dataset: 'sashelp.zipcode');
z.definekey('zip');
z.definedata('city', 'statecode', 'x', 'y');
z.definedone();
do until(last);
   set zips end=last;
   if z.find() eq 0 then output;
end;
stop;
run;

%p(z7);

%c(z7);

* temporary array;
data z8;
array z(99999) $60 _temporary_ (99999*' ');
do until(last1);
  set sashelp.zipcode (keep=zip city statecode x y) end=last1;
  z(zip) = catx(',',city, statecode,x,y);
end;
do until(last2);
   set zips end=last2;
   city = scan(z(zip),1,',');
   statecode = scan(z(zip),2,',');
   x = input(scan(z(zip),3,','),12.);
   y = input(scan(z(zip),4,','),12.);
   output;
end;
run;

%p(z8);
